
(*
    This file is part of DynLoader.

    DynLoader - cross-platform dynamic module loader.
    Author: Romiras (Feb, 2010)
    License: GPL v2 license
*)

(* Linux implementation of DynLoader *)

{$IFDEF FPC}
    {$MODE DELPHI}
{$ENDIF}

Unit HostDynLoader;

Interface

Uses Types, DynLoader, dl;

Type
    // Handle to a loaded .so
    TModuleHandle = Pointer;

    TDynLoadModule = class (TDynLoadModuleBase)
        ModuleHandle: TModuleHandle;

        // load the .so file FileName
        // the rules for FileName are those of dlopen()
        // Returns: True = success, False = failure to load
        // Assigns: the handle of the loaded .so to Module
        function LoadModule (FileName: TModuleName): Boolean; override;

        // load the .so file FileName
        // dlopen() with flags is used to get better control of the loading
        // for the allowed values for flags see "man dlopen".
        function LoadModuleEx (FileName: TModuleName; Flags: Cardinal): Boolean; override;
        
        // unload a .so loaded with LoadModule or LoadModuleEx
        // The procedure will not try to unload a handle with
        // value INVALID_MODULEHANDLE_VALUE and assigns this value
        // to Module after unload.    
        procedure UnloadModule; override;
    
        // returns the pointer to the symbol named SymbolName
        // if it is exported from the .so Module
        // nil is returned if the symbol is not available
        function GetModuleSymbol (SymbolName: TSymbolName): Pointer; override;

        // returns the pointer to the symbol named SymbolName
        // if it is exported from the .so Module
        // nil is returned if the symbol is not available.
        // as an extra the boolean variable Accu is updated
        // by anding in the success of the function.
        // This is very handy for rendering a global result
        // when accessing a long list of symbols.
        function GetModuleSymbolEx (SymbolName: TSymbolName; var Accu: Boolean): Pointer; override;
    end;

Const
    // Value designating an unassigned TModuleHandle of a failed loading
    INVALID_MODULEHANDLE_VALUE = TModuleHandle (Nil);

Implementation

function TDynLoadModule.LoadModule (FileName: TModuleName): Boolean;
begin
    ModuleHandle := dlopen (FileName, RTLD_LAZY);
    Result := ModuleHandle <> INVALID_MODULEHANDLE_VALUE
end;

function TDynLoadModule.LoadModuleEx (FileName: TModuleName; Flags: Cardinal): Boolean;
begin
    ModuleHandle := dlopen (FileName, Flags);
    Result := ModuleHandle <> INVALID_MODULEHANDLE_VALUE;
end;

procedure TDynLoadModule.UnloadModule;
begin
    if ModuleHandle <> INVALID_MODULEHANDLE_VALUE then
        dlclose (ModuleHandle);
    ModuleHandle := INVALID_MODULEHANDLE_VALUE
end;

function TDynLoadModule.GetModuleSymbol (SymbolName: TSymbolName): Pointer;
begin
    if ModuleHandle <> INVALID_MODULEHANDLE_VALUE then
        Result := dlsym (ModuleHandle, SymbolName)
    else
        Result := Nil
end;

function TDynLoadModule.GetModuleSymbolEx (SymbolName: TSymbolName; var Accu: Boolean): Pointer;
begin
    if ModuleHandle <> INVALID_MODULEHANDLE_VALUE then
        Result := dlsym (ModuleHandle, SymbolName)
    else
        Result := Nil;
    Accu := Accu And (Result <> Nil)
end;

end.
